// Elements.cpp : implementation file
//

#include "stdafx.h"
#include <cmath>
#include "Sketcher.h"
#include "Elements.h"


// CElement

CElement::CElement()
	: m_PenWidth(1)
{
}

CElement::~CElement()
{
}


// CElement member functions


CLine::CLine(void)
{
}


CLine::~CLine(void)
{
}


CCircle::CCircle(void)
{
}


CCircle::~CCircle(void)
{
}


CRectangle::CRectangle(void)
{
}


CRectangle::~CRectangle(void)
{
}


CCurve::CCurve(void)
{
}


CCurve::~CCurve(void)
{
}


// Virtual draw operation
void CElement::Draw(CDC* pDC)
{
}


// Get the bounding rectangle for an element
CRect CElement::GetBoundRect(void)
{
	CRect boundingRect(m_EnclosingRect);	// Object to store bounding rectangle

	// Increase the rectangle by the pen width and return it
	boundingRect.InflateRect(m_PenWidth, m_PenWidth);
	return boundingRect;
}

// Draw a CLine object
void CLine::Draw(CDC* pDC)
{
   // Create a pen for this object and
   // initialize it to the object color and line width of 1 pixel
   CPen aPen;
   if(!aPen.CreatePen(PS_SOLID, m_PenWidth, m_Color))
   {
      // Pen creation failed. Abort the program
      AfxMessageBox(_T("Pen creation failed drawing a line"), MB_OK);
      AfxAbort();
   }

   CPen* pOldPen = pDC->SelectObject(&aPen);  // Select the pen

   // Now draw the line
   pDC->MoveTo(m_StartPoint);
   pDC->LineTo(m_EndPoint);

   pDC->SelectObject(pOldPen);                // Restore the old pen
}

CLine::CLine(const CPoint& start, const CPoint& end, COLORREF aColor)
{
	m_StartPoint = start;
	m_EndPoint = end;
	m_Color = aColor;
	m_PenWidth = 1;		// Set pen width  

	// Define the enclosing rectangle
	m_EnclosingRect = CRect(start, end);
	m_EnclosingRect.NormalizeRect();
}

CRectangle::CRectangle(const CPoint& start, const CPoint& end, COLORREF aColor)
{
	m_Color = aColor;	// Set rectangle color
	m_PenWidth = 1;		// Set pen width

	// Define the enclosing rectangle
	m_EnclosingRect = CRect(start, end);
	m_EnclosingRect.NormalizeRect();
}

void CRectangle::Draw(CDC* pDC)
{
   // Create a pen for this object and
   // initialize it to the object color and line width of 1 pixel
   CPen aPen;
   if(!aPen.CreatePen(PS_SOLID, m_PenWidth, m_Color))
   {
      // Pen creation failed. Abort the program
      AfxMessageBox(_T("Pen creation failed drawing a line"), MB_OK);
      AfxAbort();
   }

   CPen* pOldPen = pDC->SelectObject(&aPen);  // Select the pen
   CBrush* pOldBrush = (CBrush*) pDC->SelectStockObject(NULL_BRUSH);		// Select the brush

   // Now draw the rectangle
   pDC->Rectangle(m_EnclosingRect);   

   pDC->SelectObject(pOldPen);              // Restore the old pen
   pDC->SelectObject(pOldBrush);				// Restore the old brush
}

CCircle::CCircle(const CPoint& start, const CPoint& end, COLORREF aColor)
{
	long radius = static_cast<long>( sqrt(
		static_cast<double>((end.x-start.x)*(end.x-start.x) + (end.y-start.y)*(end.y-start.y)) ));
	// Now calculate the rectangle enclosing the circle
	// assuming the MM_TEXT mapping mode
	m_EnclosingRect = CRect(start.x-radius, start.y-radius, start.x+radius, start.y+radius);
	m_EnclosingRect.NormalizeRect();

	m_Color = aColor;
	m_PenWidth = 1;
}

void CCircle::Draw(CDC* pDC)
{
   // Create a pen for this object and
   // initialize it to the object color and line width of 1 pixel
   CPen aPen;
   if(!aPen.CreatePen(PS_SOLID, m_PenWidth, m_Color))
   {
      // Pen creation failed. Abort the program
      AfxMessageBox(_T("Pen creation failed drawing a line"), MB_OK);
      AfxAbort();
   }

   CPen* pOldPen = pDC->SelectObject(&aPen);  // Select the pen
   CBrush* pOldBrush = (CBrush*) pDC->SelectStockObject(NULL_BRUSH);		// Select the brush

   // Now draw the rectangle
   pDC->Ellipse(m_EnclosingRect);   

   pDC->SelectObject(pOldPen);              // Restore the old pen
   pDC->SelectObject(pOldBrush);				// Restore the old brush
}

CCurve::CCurve(const CPoint& first, const CPoint& second, COLORREF aColor)
{
	m_Points.push_back(first);
	m_Points.push_back(second);
	m_Color = aColor;
	m_PenWidth = 1;
	m_EnclosingRect = CRect(min(first.x, second.x), min(first.y, second.y),
							max(first.x, second.x), max(first.y, second.y));
}

void CCurve::Draw(CDC* pDC)
{
   // Create a pen for this object and
   // initialize it to the object color and line width of 1 pixel
   CPen aPen;
   if(!aPen.CreatePen(PS_SOLID, m_PenWidth, m_Color))
   {
      // Pen creation failed. Abort the program
      AfxMessageBox(_T("Pen creation failed drawing a line"), MB_OK);
      AfxAbort();
   }

   CPen* pOldPen = pDC->SelectObject(&aPen);  // Select the pen
   
   // Now draw the curve
   pDC->MoveTo(m_Points[0]);
   for (size_t i=1; i<m_Points.size(); ++i)
	   pDC->LineTo(m_Points[i]);
   
   pDC->SelectObject(pOldPen);              // Restore the old pen   
}

// Add a segment to the curve
void CCurve::AddSegment(const CPoint& point)
{
	m_Points.push_back(point);

	m_EnclosingRect = CRect( min(point.x, m_EnclosingRect.left), min(point.y, m_EnclosingRect.top),
							max(point.x, m_EnclosingRect.right), max(point.y, m_EnclosingRect.bottom));
}
