#pragma once
#include "atltypes.h"
#include <vector>

// CElement command target

class CElement : public CObject
{
public:
	virtual ~CElement();
protected:
	int m_PenWidth;		// Pen width	
	COLORREF m_Color;	// Color of an element
	CElement();	
public:
	// Virtual draw operation
	virtual void Draw(CDC* pDC);
	// Get the bounding rectangle for an element
	CRect GetBoundRect(void);
protected:
	// Rectangle enclosing an element
	CRect m_EnclosingRect;
};

class CLine :	public CElement
{
public:
	virtual ~CLine(void);
	virtual void Draw(CDC* pDC);	// Function to display a line

	// Constructor for a line object
	CLine(const CPoint& start, const CPoint& end, COLORREF aColor);

protected:
	CPoint m_StartPoint;			// Start point of line
	CPoint m_EndPoint;				// End point of line
	CLine(void);					// Default constructor should not be used
};

class CCircle :
	public CElement
{
public:
	virtual ~CCircle(void);
	virtual void Draw(CDC* pDC);	// Function to display a circle

	// Constructor for a circle object
	CCircle(const CPoint& start, const CPoint& end, COLORREF aColor);

protected:
	CCircle(void);					// Default constructor - should not be used
};

class CRectangle :
	public CElement
{
public:
	virtual void Draw(CDC* pDC);	// Function to display a rectangle
	virtual ~CRectangle(void);
	// Constructor for a rectangle object
	CRectangle(const CPoint& start, const CPoint& end, COLORREF aColor);
protected:
	CRectangle(void);				// Default constructor - should not be used
};

class CCurve :
	public CElement
{
public:
	virtual ~CCurve(void);
	virtual void Draw(CDC* pDC);		// Function to display a curve

	// Constructor for a curve object
	CCurve(const CPoint& first, const CPoint& second, COLORREF aColor);
	void AddSegment(const CPoint& point);	// Add a segment to the curve
protected:
	CCurve(void);
	std::vector<CPoint> m_Points;		// Points defining the curve
};


