
// SketcherView.cpp : implementation of the CSketcherView class
//

#include "stdafx.h"
// SHARED_HANDLERS can be defined in an ATL project implementing preview, thumbnail
// and search filter handlers and allows sharing of document code with that project.
#ifndef SHARED_HANDLERS
#include "Sketcher.h"
#endif

#include "SketcherDoc.h"
#include "SketcherView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CSketcherView

IMPLEMENT_DYNCREATE(CSketcherView, CView)

BEGIN_MESSAGE_MAP(CSketcherView, CView)
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, &CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, &CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, &CSketcherView::OnFilePrintPreview)
	ON_WM_CONTEXTMENU()
	ON_WM_RBUTTONUP()
	ON_WM_LBUTTONUP()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
END_MESSAGE_MAP()

// CSketcherView construction/destruction

CSketcherView::CSketcherView()
	: m_FirstPoint(CPoint(0,0))
	, m_SecondPoint(CPoint(0,0))
	, m_pTempElement(NULL)
{
	// TODO: add construction code here

}

CSketcherView::~CSketcherView()
{
}

BOOL CSketcherView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

// CSketcherView drawing

void CSketcherView::OnDraw(CDC* pDC)
{
	CSketcherDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	if (!pDoc)
		return;	
}


// CSketcherView printing


void CSketcherView::OnFilePrintPreview()
{
#ifndef SHARED_HANDLERS
	AFXPrintPreview(this);
#endif
}

BOOL CSketcherView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CSketcherView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CSketcherView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

void CSketcherView::OnRButtonUp(UINT /* nFlags */, CPoint point)
{
	ClientToScreen(&point);
	OnContextMenu(this, point);
}

void CSketcherView::OnContextMenu(CWnd* /* pWnd */, CPoint point)
{
#ifndef SHARED_HANDLERS
	theApp.GetContextMenuManager()->ShowPopupMenu(IDR_POPUP_EDIT, point.x, point.y, this, TRUE);
#endif
}


// CSketcherView diagnostics

#ifdef _DEBUG
void CSketcherView::AssertValid() const
{
	CView::AssertValid();
}

void CSketcherView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CSketcherDoc* CSketcherView::GetDocument() const // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CSketcherDoc)));
	return (CSketcherDoc*)m_pDocument;
}
#endif //_DEBUG


// CSketcherView message handlers


void CSketcherView::OnLButtonUp(UINT nFlags, CPoint point)
{	
	// Make sure there is an element
	if (m_pTempElement)
	{
		// Call a document class function to store the element
		// pointed to by m_pTempElement in the document object

		delete m_pTempElement;
		m_pTempElement = 0;		
	}
}


void CSketcherView::OnLButtonDown(UINT nFlags, CPoint point)
{
	// TODO: Add your message handler code here and/or call default

	m_FirstPoint = point;
	
}


void CSketcherView::OnMouseMove(UINT nFlags, CPoint point)
{  // Define a Device Context object for the view
   CClientDC aDC(this);                // DC is for this view
   if((nFlags & MK_LBUTTON) )
   {  m_SecondPoint = point;           // Save the current cursor position
      if(m_pTempElement)
      {  
		  if (CURVE == GetDocument()->GetElementType())		// Is it a curve?
		  {  // We are drawing a curve so add a segment to the existing curve
			  static_cast<CCurve*>(m_pTempElement)->AddSegment(m_SecondPoint);
			  m_pTempElement->Draw(&aDC);					// Now draw it
			  return;										// We are done
		  }

		  // Redraw the old element so it disappears from the view
         aDC.SetROP2(R2_NOTXORPEN);          // Set the drawing mode
		 m_pTempElement->Draw(&aDC);
         delete m_pTempElement;        // Delete the old element
         m_pTempElement = NULL;           // Reset the pointer to 0
      }
      // Create a temporary element of the type and color that
      // is recorded in the document object, and draw it
      m_pTempElement = CreateElement();// Create a new element	 
      m_pTempElement->Draw(&aDC);      // Draw the element
   }	
}


// Create a new element on the heap
CElement* CSketcherView::CreateElement(void)
{
	// Get a pointer to the document for this view
   CSketcherDoc* pDoc = GetDocument();
   
   // Now select the element using the type stored in the document
   switch(pDoc->GetElementType())
   {
		case RECTANGLE:
         return new CRectangle(m_FirstPoint, m_SecondPoint, 
			pDoc->GetElementColor());

		case CIRCLE:
         return new CCircle(m_FirstPoint, m_SecondPoint, 
			pDoc->GetElementColor());

		case LINE:
         return new CLine(m_FirstPoint, m_SecondPoint, 
			pDoc->GetElementColor());
		case CURVE:
			return new CCurve(m_FirstPoint, m_SecondPoint, pDoc->GetElementColor());

      default:
         // Something's gone wrong
         AfxMessageBox(_T("Bad Element code"), MB_OK);
	     AfxAbort();
         return NULL;
	}
}
